@@ -159,7 +159,7 @@ pub enum CmsError {
     Pem(PemError),
 
     /// Error occurred when creating a signature.
-    SignatureCreation,
+    SignatureCreation(signature::Error),
 
     /// Attempted to use a `Certificate` but we couldn't find the backing data for it.
     CertificateMissingData,
@@ -217,7 +217,9 @@ impl Display for CmsError {
             Self::NoSignedAttributes => f.write_str("SignedAttributes structure is missing"),
             Self::DigestNotEqual => f.write_str("digests not equivalent"),
             Self::Pem(e) => f.write_fmt(format_args!("PEM error: {}", e)),
-            Self::SignatureCreation => f.write_str("error during signature creation"),
+            Self::SignatureCreation(e) => {
+                f.write_fmt(format_args!("error during signature creation: {}", e))
+            }
             Self::CertificateMissingData => f.write_str("certificate data not available"),
             Self::DistinguishedNameParseError => {
                 f.write_str("could not parse distinguished name data")
@@ -256,6 +258,12 @@ impl From<TimeStampError> for CmsError {
     }
 }
 
+impl From<signature::Error> for CmsError {
+    fn from(e: signature::Error) -> Self {
+        Self::SignatureCreation(e)
+    }
+}
+
 impl From<X509CertificateError> for CmsError {
     fn from(e: X509CertificateError) -> Self {
         Self::X509Certificate(e)
@@ -921,6 +929,7 @@ impl TryFrom<&crate::asn1::rfc5652::SignerInfo> for SignerInfo {
                 content_type,
                 message_digest,
                 signing_time,
+                raw: attributes.clone(),
             })
         } else {
             None
@@ -978,6 +987,9 @@ pub struct SignedAttributes {
 
     /// The time the signature was created.
     signing_time: Option<chrono::DateTime<chrono::Utc>>,
+
+    /// The raw ASN.1 signed attributes.
+    raw: crate::asn1::rfc5652::SignedAttributes,
 }
 
 impl SignedAttributes {
@@ -992,6 +1004,10 @@ impl SignedAttributes {
     pub fn signing_time(&self) -> Option<&chrono::DateTime<chrono::Utc>> {
         self.signing_time.as_ref()
     }
+
+    pub fn attributes(&self) -> &crate::asn1::rfc5652::SignedAttributes {
+        &self.raw
+    }
 }
 
 impl Debug for SignedAttributes {
